#include <memory>
#include <sstream>

#include "types1516.h"

// IEEE 1516.2, Section 4.12.9.1
void test1()
{
  typedef
#define FIELD_A 0
    HLAfixedRecord<HLAoctet,
#define FIELD_B 1
    HLAfixedRecord<HLAboolean,
#define FIELD_C 2
    HLAfixedRecord<HLAfloat64BE,
    HLAfixedRecordEnd> > > TA;

  HLAbuffer<TA> A;
  (*A).field<FIELD_A>() = 'A';
  (*A).field<FIELD_B>() = HLAtrue;
  (*A).field<FIELD_C>() = 3.14;

  std::stringstream result;
  A.print(result);

  const char* correct =
    "0000:  41 00 00 00 00 00 00 01 40 09 1e b8 51 eb 85 1f\n";

  if(strcmp(result.str().c_str(), correct) != 0)
  {
    std::cerr << "test1: <output> does not match expected result" << std::endl
      << result.str() << std::endl << correct << std::endl;
  }
  else
    std::cout << result.str();
}

// TODO: Add support for HLAvariantRecord

// IEEE 1516.2, Section 4.12.9.3
void test3()
{
  typedef
#define FIELD_A 0
    HLAfixedRecord<HLAinteger32BE,
#define FIELD_B 1
    HLAfixedRecord<HLAoctet,
    HLAfixedRecordEnd> > TA;

  typedef HLAfixedArray<TA,2> TB;

  HLAbuffer<TB> B;
  (*B)[0].field<FIELD_B>() = '0';
  (*B)[1].field<FIELD_A>() = 42;
  (*B)[1].field<FIELD_B>() = '1';

  std::stringstream result;
  B.print(result);

  const char* correct =
    "0000:  00 00 00 00 30 00 00 00 00 00 00 2a 31\n";

  if(strcmp(result.str().c_str(), correct) != 0)
  {
    std::cerr << "test2: <output> does not match expected result" << std::endl
      << result.str() << std::endl << correct << std::endl;
  }
  else
    std::cout << result.str();
}

// IEEE 1516.2, Section 4.12.9.4
void test4()
{
  typedef HLAvariableArray<HLAfloat64BE> TA;

  HLAbuffer<TA> A;
  (*A).set_size(1);
  (*A)[0] = 3.14;

  std::stringstream result;
  A.print(result);

  const char* correct =
    "0000:  00 00 00 01 00 00 00 00 40 09 1e b8 51 eb 85 1f\n";

  if(strcmp(result.str().c_str(), correct) != 0)
  {
    std::cerr << "test4: <output> does not match expected result" << std::endl
      << result.str() << std::endl << correct << std::endl;
  }
  else
    std::cout << result.str();
}

void test5()
{
  typedef
    HLAvariableArray<
#define FIELD_A 0
      HLAfixedRecord<HLAvariableArray<HLAfloat64LE>,
#define FIELD_B 1
      HLAfixedRecord<HLAvariableArray<HLAinteger32LE>,
      HLAfixedRecordEnd> > > TA;

  HLAbuffer<TA> A;
  (*A).set_size(1);
  (*A)[0].field<FIELD_A>().set_size(1);
  (*A)[0].field<FIELD_A>()[0] = 3.14;
  (*A)[0].field<FIELD_B>().set_size(1);
  (*A)[0].field<FIELD_B>()[0] = 7;

  (*A)[0].field<FIELD_A>().set_size(2);
  (*A)[0].field<FIELD_A>()[1] = 2.718;

  std::stringstream result;
  A.print(result);

  const char* correct =
    "0000:  00 00 00 01 00 00 00 00 00 00 00 02 00 00 00 00\n"
    "0010:  1f 85 eb 51 b8 1e 09 40 58 39 b4 c8 76 be 05 40\n"
    "0020:  00 00 00 01 07 00 00 00\n";

  if(strcmp(result.str().c_str(), correct) != 0)
  {
    std::cerr << "test1: <output> does not match expected result" << std::endl
      << result.str() << std::endl << correct << std::endl;
  }
  else
    std::cout << result.str();
}

int main(int argc, char* argv[])
{
  test1();
  test3();
  test4();
  test5();

  return 0;
}

